module Dgrid
  module API
    class Entity
      include ::Dgrid::ArgumentValidation
      include ::Dgrid::SetMembersFromHash
      
      attr_accessor :id, :description

      option :id, String
      option :description, String, :default => ''
      def initialize(options)
        @id = nil
        set_members_from_hash(options)

        # FIXME Remove this when ArgumentValidation::option is implemented
        @description ||= ''

        @workspaces = {}
      end

      def self.type
        self.name.split('::').last
      end

      def type
        self.class.type
      end

      def self.singular
        name.split('::').last.downcase
      end


      argument :other, Entity
      option :link_type, String, :required
      def link_to(other, options = {})
        raise UnimplementedFunctionality
        self
      end

      def in_workspace?(workspace)
        return @workspaces.include?(workspace.id)
      end

      def add_workspace(workspace)
        @workspaces[workspace.id] = workspace
        self
      end

      def to_hash
        { :description => description }
      end

      def new_record?
        @id.nil?
      end

      def self.pluralized
        'entities'
      end

      def add_entity(entity)
        workspaces.each do |workspace_id,workspace|
          # STDERR.puts "adding #{entity.type} #{entity} to #{self.type} #{self} in workspace #{workspace}"
          if entity.in_workspace?(workspace)
            workspace.subordinate_entity_to_other_entity(entity, self)
          end
        end
      end

      def attach(filename)
        conn = connection
        # TODO Adapt this to a workspace-independent model when available in the server
        conn.attach_file_to_entity_in_workspace(self,filename,first_workspace.id)
      end

      def connection
        first_workspace.connection
      end

      def first_workspace
        workspaces.values.first
      end

      attr_accessor :workspaces

    end
  end
end
