module Dgrid
  module API
    class Incident < NamedEntity
      attr_accessor :body, :time, :end_time
      
      option :name, String, :required     # unique for this workspace 
      option :description, String
      option :body, String
      option :time, DateTime
      option :end_time, DateTime

      # alternative way to specify times from server
      option :time_date, String
      option :time_time, String
      option :end_time_date, String
      option :end_time_time, String
      def initialize(options)
        parent_opts, my_opts = split_hash(options,[:name, :description])
        super(parent_opts)
        member_opts = fix_time_opts(my_opts)
        set_members_from_hash(member_opts)
      end

      def self.db_fields
        %w(id name body time end_time)
      end

      def add_time_to_hash(h, t,prefix)
        h["#{prefix}_time".to_sym]  = t.strftime("%H:%M")
        h["#{prefix}_date".to_sym] = t.strftime("%Y-%m-%d")
        h
      end

      def self.pluralized
        'incidents'
      end

      def to_hash
        h = { :name => name,
              :body => body
            }
        add_time_to_hash(h,time,'time') unless time.nil?
        add_time_to_hash(h,end_time,'end_time') unless end_time.nil?
        h
      end

      argument :person, Dgrid::API::Person
      def add_person(person)
        add_entity(person)
      end

      argument :place, Dgrid::API::Place
      def add_place(place)
        add_entity(place)
      end

      argument :organization, Dgrid::API::Organization
      def add_organization(organization)
        add_entity(organization)
      end

      protected
        def fix_time_opts(orig_opts)
          split_time_opts, normal_opts = split_hash(orig_opts,[:time_time,:time_date,:end_time_time, :end_time_date])
          [:time, :end_time].each do |member|
             date_part = (member.to_s + "_date").to_sym
             time_part = (member.to_s + "_time").to_sym
             if (split_time_opts.include?(date_part)) 
                 normal_opts[member] = split_time_opts[date_part] + " " + split_time_opts[time_part]
             end
          end
           normal_opts[:time] = DateTime::parse( normal_opts[:time]) if  normal_opts[:time].is_a?(String)
           normal_opts[:end_time] = DateTime::parse( normal_opts[:end_time]) if normal_opts[:end_time].is_a?(String)
           normal_opts
        end
    end
  end
end
