module Dgrid
  module API
    class Item < NamedEntity
      include ArgumentValidation
      attr_accessor :body
      
      option :name, String, :required     # unique for this workspace 
      option :body, String
      option :source, String
      def initialize(options)
        parent_opts, my_opts = split_hash(options,[:name, :description])
        super(parent_opts)
        set_members_from_hash(my_opts)
        @body ||= ''
        @incidents = nil
      end

      def self.db_fields
        %w(id name body source)
      end
      
      def self.pluralized
        'items'
      end

      def to_hash
        { :name => name,
          :body => body
        }
      end

      def workspace
        workspace = workspaces.values.first # FIXME: assumes one workspace per item
      end

      def incident_ids
        workspace.incident_ids_in_item(self)
      end

      def incidents
        incident_list = workspace.incidents
        incident_hash = {}
        incident_list.each { |incident| incident_hash[incident.id] = incident}
        return incident_ids.map { |id| incident_hash[id]}
      end


      def create_incident(options = {}, &block)
        incident = Dgrid::API::Incident.new(options)
        workspaces.each do |workspace_id, workspace| 
          returned_incident = workspace.add_incident(incident)
          incident = returned_incident if incident.new_record?
          workspace.subordinate_entity_to_other_entity(incident,self)
        end
        yield incident if block_given?
        incident
      end
    end
  end
end
