module Dgrid
  module API
    class Place < NamedEntity
      attr_accessor :lat_long

      option :name, String, :required     # unique for this workspace 
      option :description, String         # typically an address
      option :lat_long, GPSCoordinates   # When known. Description will be geocoded if this is not supplied.
      option :latitude, String           # alternative input method for db restoration
      option :longitude, String
      option :elevation, String
      def initialize(options)
      	parent_opts, my_opts = split_hash(options,[:name, :description])
      	super(parent_opts)
        gps_opts, other_opts = split_hash(options,[:latitude, :longitude, :elevation])
      	set_members_from_hash(other_opts)
        if !other_opts.include?(:lat_long) && !gps_opts.empty?
          self.lat_long = coalesce_gps_opts(gps_opts)
        end
      end


      def self.db_fields
        %w(id name description latitude longitude elevation)
      end

      def self.pluralized
        'places'
      end
      
      def to_hash
        h = super
        if lat_long
          h.merge!({:latitude => lat_long.lat, :longitude => lat_long.long, :elevation => lat_long.elevation})
        end
        h
      end

      protected
        def coalesce_gps_opts(gps_opts)
          gps_opts[:elevation] ||= 0
          gps_args = [:latitude,:longitude,:elevation].map {|k| gps_opts[k]}
          GPSCoordinates.new(*gps_args)
        end
    end
  end
end
